#pragma once

namespace limax {

#ifdef LIMAX_OS_UNIX_FAMILY
	typedef int SOCKET;
#endif
	class TcpClient
	{
	public:
		struct Listener
		{
			virtual ~Listener() { }
			virtual void onCreate(TcpClient *) = 0;
			virtual void onOpen(const struct sockaddr_in& local, const struct sockaddr_in& peer) = 0;
			virtual void onAbort(const struct sockaddr_in &sa) = 0;
			virtual void onRecv(const void *data, int32_t size) = 0;
			virtual void onClose(int status) = 0;
		};
	private:
		SOCKET sock;
		struct sockaddr_in local, peer;
		Listener* listener;
		std::atomic<bool> closed;
		std::atomic<bool> close_notified;
		BlockingQueue<Octets> sq;
		JoinableRunnable r_task;
		JoinableRunnable w_task;
		void _close();
		void _connected();
		void _abort();
		~TcpClient();
		TcpClient(Listener* l, std::string ip, short port);
		TcpClient(const TcpClient&) = delete;
		TcpClient& operator= (const TcpClient&) = delete;
	public:
		static void create(Listener* l, std::string ip, short port);
		void send(Octets o);
		void close();
		void destroy();
	};
} // namespace limax {

