package limax.auany;

import java.util.LinkedHashMap;
import java.util.Map;
import java.util.concurrent.Future;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import org.w3c.dom.Element;

import limax.auany.switcherauany.SessionAuthByToken;
import limax.switcherauany.AuanyAuthArg;
import limax.switcherauany.AuanyAuthRes;
import limax.util.ConcurrentEnvironment;
import limax.util.ElementHelper;

public final class LoginInfoCache {
	private static int capacity;
	private static long lifetime;
	private static ScheduledExecutorService houseKeeper;

	private static class Key {
		private final String username;
		private final String platflag;

		Key(String username, String platflag) {
			this.username = username;
			this.platflag = platflag;
		}

		@Override
		public boolean equals(Object o) {
			Key k = (Key) o;
			return username.equals(k.username) && platflag.equals(k.platflag);
		}

		@Override
		public int hashCode() {
			return username.hashCode() ^ platflag.hashCode();
		}
	}

	private static class Record {
		private final Future<?> future;
		private final SessionAuthByToken rpc;

		Record(Key key, SessionAuthByToken rpc) {
			Record o = cache.put(key, this);
			if (o != null)
				o.cancel();
			this.rpc = rpc;
			this.future = houseKeeper.schedule(() -> cache.remove(key), lifetime, TimeUnit.MILLISECONDS);
		}

		void cancel() {
			future.cancel(false);
		}
	}

	private final static Map<Key, Record> cache = new LinkedHashMap<Key, Record>() {
		private static final long serialVersionUID = -8651921173899841939L;

		protected boolean removeEldestEntry(Map.Entry<Key, Record> eldest) {
			if (size() > capacity) {
				eldest.getValue().cancel();
				return true;
			}
			return false;
		}
	};

	static void initialize(Element self) {
		ElementHelper eh = new ElementHelper(self);
		capacity = eh.getInt("loginInfoCacheCapacity", 8192);
		lifetime = eh.getLong("loginInfoCacheLifetime", 300000l);
		houseKeeper = ConcurrentEnvironment.getInstance().newScheduledThreadPool("LoginInfoCacheHouseKeeper", 1);
	}

	static synchronized void unInitialize() {
		cache.values().forEach(v -> v.cancel());
		cache.clear();
		ConcurrentEnvironment.getInstance().shutdown("LoginInfoCacheHouseKeeper");
	}

	static synchronized void add(SessionAuthByToken rpc) {
		new Record(new Key(rpc.getArgument().username, rpc.getArgument().platflag), rpc);
	}

	static AuanyAuthRes tryLogin(AuanyAuthArg arg) {
		SessionAuthByToken rpc = get(arg.username, arg.platflag);
		if (rpc == null || !arg.equals(rpc.getArgument()))
			return null;
		return rpc.getResult();
	}

	public static synchronized SessionAuthByToken get(String username, String platflag) {
		Record r = cache.get(new Key(username, platflag));
		return r == null ? null : r.rpc;
	}
}
